#@ String (value = "Astrocyte Territory" ) astro_str
#@ String (value = "AQP-4") aqporine_str
// to make clusters from identified surfaces of end-feet
#@ Double  (value = 2.5) cluster_max_dist // distance in micron
#@ String (choices={"Ball (very slow)" , "Cube "} ) strel_shape
#@ ResultsTable rt
Prefs.blackBackground = true;
showImages = true
rt.reset()
IJ.run("Close All", "");


/* = CODE DESCRIPTION =
 *  This script generates Endfeet Imaris surfaces and a new channel containing cluster information. 
 *  To obtain the Endfeet Imaris surfaces, the mask images of the Astrocyte Territory and AQP4 Imaris 
 *surfaces were processed using an AND operator to generate the Endfeet mask. The Endfeet mask image 
 * was converted into an Imaris Surfaces object and added to the Imaris Scene to generate the Endfeet 
 * Imaris surfaces. Additionally, the Endfeet mask image was dilated using a spherical element with 
 * a radius of ….[y], and a 3D connected components analysis was performed [y]. The resulting Label image 
 * was included in the Imaris dataset as a channel, containing clustering information.
 *
 * == INPUTS ==
 * - An Imaris file open in Imaris with surfaces of "Astrocyte Territory" and "AQP-4"
 * - the maximal distance to consider surface to be part of the same clsuter : cluster_max_dist
 * - the element to use for 3D dilation Ball or Cube
 *
 * == OUTPUTS ==
 * New Imaris Surfaces Endfoot and a cluster channel
 * Fiji results table 
 *
 * = DEPENDENCIES =
 * Imaris
 * EasyXT
 * MorphologyJ
 *
 * = INSTALLATION =
 * Imaris installer 
 * Fiji update sites :
 * - PTBIOP
 * - IJPB-plugins
 *
 * = AUTHOR INFORMATION =
 * Code written by Romain Guiet , EPFL - SV - PTECH - BIOP
 * for Charlotte Lorin, Lab LNMC
 * 2023.04.14
 *
 * = COPYRIGHT =
 * (c) All rights reserved. ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE, Switzerland, BioImaging And Optics Platform (BIOP), 2023
 *
 * Licensed under the BSD-3-Clause License:
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer
 *    in the documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

EasyXT.Utils.resetImarisConnection()
// prepare for output
results_dir = new File ( EasyXT.Files.getOpenFile().getParent() , "results")
results_dir.mkdir()
results_path = new File ( results_dir , EasyXT.Files.getOpenFileName()+ "_results.csv")

// From defined name, get surfaces and corresponding masks
def astro_surf = EasyXT.Surfaces.find(astro_str)
def aqporine_surf = EasyXT.Surfaces.find(aqporine_str)

def astro_imp = EasyXT.Surfaces.getMaskImage(astro_surf)
def aqporine_imp = EasyXT.Surfaces.getMaskImage(aqporine_surf)

// we'll need calibration later to import back in Imaris
def cal = astro_imp.getCalibration()

// COMPUTE AND between masks
def aqporine_AND_astro_imp= ImageCalculator.run(astro_imp, aqporine_imp, "AND create stack");
aqporine_AND_astro_imp.setTitle(astro_str + "_AND_" + aqporine_str)

def endFeetEstimate_label = connectedComponent3d(aqporine_AND_astro_imp)
endFeetEstimate_label.setCalibration(cal)
def endFeetEstimate_surf = EasyXT.Surfaces.createFromLabels(endFeetEstimate_label)
EasyXT.Scene.setName(endFeetEstimate_surf, "Endfoot")
EasyXT.Scene.addItem(endFeetEstimate_surf)

// to identify objects that are at a maximum of "cluster_max_dist"
// we take half of that value
half_way = cluster_max_dist / 2  
// then use calibration information to get value in "pixels"
half_way_pixel = half_way / cal.pixelWidth as int // get number of iteration
anisotropy = cal.pixelDepth / cal.pixelWidth as int

def mf3Dp = new  MorphologicalFilter3DPlugin()
def op = Operation.DILATION
def strel = CuboidStrel.fromRadiusList(1, 1, anisotropy)
if (  !strel_shape.equals ("Cube") ){	strel = EllipsoidStrel.fromRadiusList(1, 1, anisotropy) } 

def dilated_imp = mf3Dp.process(aqporine_AND_astro_imp , op , strel )
// continue with itrations 
(0..half_way_pixel-2).each{	dilated_imp = mf3Dp.process(dilated_imp , op , strel )}

// so we can measure to which cluster belongs surfaces
def cluster_label = connectedComponent3d( dilated_imp )
EasyXT.Dataset.addChannels( cluster_label )

rt = EasyXT.Stats.export( endFeetEstimate_surf );           
                                                   
rt.show("Results");
IJ.saveAs("Results", results_path.toString() );

println "End-Feet estimation computed" 


if (showImages){
	astro_imp.show()
	aqporine_imp.show()
	aqporine_AND_astro_imp.show()
	endFeetEstimate_label.show()
	dilated_imp.show()
	cluster_label.show()
	IJ.run("Tile", "");
}

/*
 * make label from binary image
 */
//import inra.ijpb.binary.conncomp.FloodFillComponentsLabeling3D;
//import ij.ImagePlus
def connectedComponent3d(imp){
	FloodFillComponentsLabeling3D algo = new FloodFillComponentsLabeling3D(6, 16);
	FloodFillComponentsLabeling3D.Result res = algo.computeResult(imp.getStack());
	labels_imp = new ImagePlus("label", res.labelMap);
	return labels_imp
}


import Imaris.*
import ch.epfl.biop.imaris.*
import ch.epfl.biop.imaris.EasyXT

import inra.ijpb.plugins.*
import inra.ijpb.morphology.*
import inra.ijpb.morphology.Morphology.Operation
import inra.ijpb.morphology.Strel3D
import inra.ijpb.morphology.strel.EllipsoidStrel
import inra.ijpb.morphology.strel.CuboidStrel
import inra.ijpb.binary.conncomp.FloodFillComponentsLabeling3D                     

import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.*;
                                                                                                         
import ij.*
import ij.ImagePlus
import ij.ImageStack
import ij.gui.Overlay
import ij.gui.PolygonRoi
import ij.plugin.Duplicator
import ij.plugin.Concatenator
import ij.plugin.ImageCalculator
import ij.plugin.HyperStackConverter